# PLAID Printing
# This stupid idea takes a PDF and parallel prints it across two printers.
# Remember to uninstall every "printer" you have beforehand, or else this will just Print To PDF and send a copy to Outlook.
# Tested on a Lenovo X1 Carbon Gen 5, with two USB attached HP Color LaserJet Pro M452nw and a generic HP PCL6 Print Driver.
# Written by Matthew Jones, with mild assistance of Copilot and Python documents. 8/2/2024
# mcjonestech.com


from PyPDF2 import PdfReader, PdfWriter
import win32print
import win32api
import multiprocessing
import os

def find_printers():
    printers = []
    for printer in win32print.EnumPrinters(win32print.PRINTER_ENUM_LOCAL):
        printers.append({'name': printer[2]})
    if not printers:
        raise Exception("No printers found")
    print("Printers found:", printers)
    return printers

def split_pdf(filepath):
    print(f"Splitting PDF: {filepath}")
    with open(filepath, "rb") as file:
        input_pdf = PdfReader(file)
        total_pages = len(input_pdf.pages)
        midpoint = total_pages // 2  # To do: Make this round. I have been only doing even number page documents, so I have not felt the need to round.
        print(f"Total pages: {total_pages}, Midpoint: {midpoint}")

        pagesA = input_pdf.pages[:midpoint]
        pagesB = input_pdf.pages[midpoint:]

        outputA = PdfWriter()
        outputB = PdfWriter()

        for page in pagesA:
            outputA.add_page(page)
        for page in pagesB:
            outputB.add_page(page)

        print("PDF split into two parts")
        return outputA, outputB

def print_page(printer, pdf_writer):
    printer_name = printer['name']
    print(f"Preparing to print on {printer_name}")
    temp_path = "temp_page.pdf" # The easiest solution I could come up with is a temporary PDF of both sides of the midpoint that is deleted afterwards.
    with open(temp_path, "wb") as temp_pdf:
        pdf_writer.write(temp_pdf)

    try:
        print_job(printer_name, temp_path)
    except Exception as e:
        print(f"Failed to print on {printer_name}: {e}")
    finally:
        os.remove(temp_path)
        print(f"Temporary file {temp_path} removed")

def print_job(printer_name, filepath):
    # Open the printer
    printer_handle = win32print.OpenPrinter(printer_name)
    try:
        # Define document info (This part will break the whole code if its not here so here it is)
        doc_info = (
            "Print Job",
            None,
            "RAW"
        )
        
        # Start the print job
        job_id = win32print.StartDocPrinter(printer_handle, 1, doc_info)
        win32print.StartPagePrinter(printer_handle)
        
        with open(filepath, "rb") as file:
            data = file.read()
            win32print.WritePrinter(printer_handle, data)
        
        win32print.EndPagePrinter(printer_handle)
        win32print.EndDocPrinter(printer_handle)
        print(f"Print job sent to {printer_name}")
    finally:
        win32print.ClosePrinter(printer_handle)



def plaid_print(printers, pdf_writers):  # Yet to see this thing work, will keep testing this nightmare. Thanks, Copilot.
    print("Starting parallel print tasks")
    
    tasks = [
        (printers[0], pdf_writers[0]),
        (printers[1], pdf_writers[1])
    ]

    try:
        with multiprocessing.Pool(processes=2) as pool:
            pool.starmap(print_page, tasks)
            pool.close()
            pool.join()
            print("Print tasks completed")
    except Exception as e:
        print(f"Error in multiprocessing: {e}")
    finally:
        print("Parallel print tasks completed")

def plaid_print_start(printers, pdf_writers):
    print("Starting print tasks")

    for printer, pdf_writer in zip(printers, pdf_writers):
        print_page(printer, pdf_writer)

    print("Print tasks completed")


# The main code, woohoo

filepath = input("Enter file path here: ")

print(f"File path entered: {filepath}")

if not os.path.isfile(filepath):
    print("File not found or is not accessible")
else:
    try:
        printers = find_printers()
        if len(printers) < 2:
            print("At least two printers are required")
        else:
            pdf_writers = split_pdf(filepath)
            plaid_print_start(printers, pdf_writers)
    except Exception as e:
        print("An error occurred:", e)
